@@ -36,7 +36,7 @@ function sendViaWS (url, payload, success, failure) {
 
 // Editor class
 
-function Editor (name, url, sanitize) {
+function Editor (name, url, sanitize, pagedown_extra_options) {
   this.url = url;   
   var data = {
     name  : name,
@@ -54,6 +54,7 @@ function Editor (name, url, sanitize) {
   var converter = sanitize
     ? Markdown.getSanitizingConverter()
     : new Markdown.Converter();
+  var extra = Markdown.Extra.init(converter, pagedown_extra_options);
 
   var editor = new Markdown.Editor(converter);
   converter.hooks.chain("preConversion", function (text) {
@@ -61,6 +62,11 @@ function Editor (name, url, sanitize) {
     return text; 
   });
   converter.hooks.chain("postConversion", function (text) {
+    // codeblock of original pagedown looks little differ rather than
+    // fenced code block of pagedown-extra so need little fix
+    var preclass = extra.googleCodePrettify ? ' class="prettyprint"' : extra.highlightJs ? ' class="hljs"' : '';
+    text = text.replace(/<pre><code>/g, "<pre" + preclass + "><code>");
+
     data.html = text;
     return text; 
   });
